@@ -84,8 +84,11 @@ static void schedule_signal __P((int signo));
 static int log_io;
 #endif /* _PATH_SUDO_IO_LOGDIR */
 
-static int handle_signals __P((int fd, pid_t child,
+static int handle_signals __P((int sv[2], pid_t child,
     struct command_status *cstat));
+#ifdef SA_SIGINFO
+static void handler_nofwd __P((int s, siginfo_t *info, void *context));
+#endif
 
 /*
  * Like execve(2) but falls back to running through /bin/sh
@@ -142,7 +145,11 @@ static int fork_cmnd(path, argv, envp, sv, rbac_enabled)
 	restore_signals();
 	if (exec_setup(rbac_enabled, user_ttypath, -1) == TRUE) {
 	    /* headed for execve() */
-	    closefrom(def_closefrom);
+	    int maxfd = def_closefrom;
+	    dup2(sv[1], maxfd);
+	    (void)fcntl(maxfd, F_SETFD, FD_CLOEXEC);
+	    sv[1] = maxfd++;
+	    closefrom(maxfd);
 #ifdef HAVE_SELINUX
 	    if (rbac_enabled)
 		selinux_execve(path, argv, envp);
@@ -292,14 +299,31 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
     sa.sa_handler = handler;
     sigaction(SIGALRM, &sa, NULL);
     sigaction(SIGCHLD, &sa, NULL);
-    sigaction(SIGHUP, &sa, NULL);
-    sigaction(SIGINT, &sa, NULL);
     sigaction(SIGPIPE, &sa, NULL);
-    sigaction(SIGQUIT, &sa, NULL);
     sigaction(SIGTERM, &sa, NULL);
     sigaction(SIGUSR1, &sa, NULL);
     sigaction(SIGUSR2, &sa, NULL);
 
+    /*
+     * When not running the command in a pty, we do not want to
+     * forward signals generated by the kernel that the child will
+     * already have received either by virtue of being in the
+     * controlling tty's process group (SIGINT, SIGQUIT) or because
+     * the session is terminating (SIGHUP).
+     */
+#ifdef SA_SIGINFO
+# ifdef _PATH_SUDO_IO_LOGDIR
+    if (!log_io)
+# endif
+    {
+	sa.sa_flags |= SA_SIGINFO;
+	sa.sa_sigaction = handler_nofwd;
+    }
+#endif /* SA_SIGINFO */
+    sigaction(SIGHUP, &sa, NULL);
+    sigaction(SIGINT, &sa, NULL);
+    sigaction(SIGQUIT, &sa, NULL);
+
     /* Max fd we will be selecting on. */
     maxfd = MAX(sv[0], signal_pipe[0]);
 
@@ -353,7 +377,7 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
 	}
 #endif /* _PATH_SUDO_IO_LOGDIR */
 	if (FD_ISSET(signal_pipe[0], fdsr)) {
-	    n = handle_signals(signal_pipe[0], child, cstat);
+	    n = handle_signals(sv, child, cstat);
 	    if (n == 0) {
 		/* Child has exited, cstat is set, we are done. */
 		goto done;
@@ -368,17 +392,23 @@ sudo_execve(path, argv, envp, uid, cstat, dowait, bgmode)
 	if (FD_ISSET(sv[0], fdsr)) {
 	    /* read child status */
 	    n = recv(sv[0], cstat, sizeof(*cstat), 0);
-	    if (n == -1) {
-		if (errno == EINTR)
-		    continue;
+	    if (n != sizeof(*cstat)) {
+		if (n == -1) {
+		    if (errno == EINTR)
+			continue;
 #ifdef _PATH_SUDO_IO_LOGDIR
-		/*
-		 * If not logging I/O we will receive ECONNRESET when
-		 * the command is executed.  It is safe to ignore this.
-		 */
-		if (log_io && errno != EAGAIN) {
-		    cstat->type = CMD_ERRNO;
-		    cstat->val = errno;
+		    /*
+		     * If not logging I/O we will receive ECONNRESET when
+		     * the command is executed.  It is safe to ignore this.
+		     */
+		    if (log_io && errno != EAGAIN) {
+			cstat->type = CMD_ERRNO;
+			cstat->val = errno;
+			break;
+		    }
+		} else {
+		    /* Short read or EOF. */
+		    /* XXX - should set cstat */
 		    break;
 		}
 #endif
@@ -446,8 +476,8 @@ done:
  * Returns -1 on error, 0 on child exit, else 1.
  */
 static int
-handle_signals(fd, child, cstat)
-    int fd;
+handle_signals(sv, child, cstat)
+    int sv[2];
     pid_t child;
     struct command_status *cstat;
 {
@@ -486,9 +516,17 @@ handle_signals(fd, child, cstat)
 #endif
 	    } while (pid == -1 && errno == EINTR);
 	    if (pid == child) {
-		/* If not logging I/O and child has exited we are done. */
 #ifdef _PATH_SUDO_IO_LOGDIR
-		if (!log_io)
+		if (log_io) {
+		    /*
+		     * On BSD we get ECONNRESET on sv[0] if monitor dies
+		     * and select() will return with sv[0] readable.
+		     * On Linux that doesn't appear to happen so if the
+		     * monitor dies, shut down the socketpair to force a
+		     * select() notification.
+		     */
+		    (void) shutdown(sv[0], SHUT_WR);
+		} else
 #endif
 		{
 		    if (WIFSTOPPED(status)) {
@@ -518,7 +556,6 @@ handle_signals(fd, child, cstat)
 			return 0;
 		    }
 		}
-		/* Else we get ECONNRESET on sv[0] if child dies. */
 	    }
 	} else {
 #ifdef _PATH_SUDO_IO_LOGDIR
@@ -566,6 +603,7 @@ forward_signals(sock)
 		    tq_remove(&sigfwd_list, sigfwd);
 		    efree(sigfwd);
 		}
+		/* XXX - child (monitor) is dead, we should exit too? */
 	    }
 	    break;
 	}
@@ -607,6 +645,33 @@ handler(s)
 	/* shut up glibc */;
 }
 
+#ifdef SA_SIGINFO
+/*
+ * Generic handler for signals passed from parent -> child.
+ * The other end of signal_pipe is checked in the main event loop.
+ * This version is for the non-pty case and does not forward
+ * signals that are generated by the kernel.
+ */
+static RETSIGTYPE
+handler_nofwd(s, info, context)
+    int s;
+    siginfo_t *info;
+    void *context;
+{
+    unsigned char signo = (unsigned char)s;
+
+    /* Only forward user-generated signals. */
+    if (info->si_code <= 0) {
+	/*
+	 * The pipe is non-blocking, if we overflow the kernel's pipe
+	 * buffer we drop the signal.  This is not a problem in practice.
+	 */
+	if (write(signal_pipe[1], &signo, sizeof(signo)) == -1)
+	    /* shut up glibc */;
+    }
+}
+#endif /* SA_SIGINFO */
+
 /*
  * Open a pipe and make both ends non-blocking.
  * Returns 0 on success and -1 on error.
