@@ -115,8 +115,12 @@ sudo_file_parse(nss)
     init_parser(_PATH_SUDOERS, 0);
     yyin = nss->handle;
     if (yyparse() != 0 || parse_error) {
-	log_error(NO_EXIT, "parse error in %s near line %d",
-	    errorfile, errorlineno);
+	if (errorlineno != -1) {
+	    log_error(NO_EXIT, "parse error in %s near line %d",
+		errorfile, errorlineno);
+	} else {
+	    log_error(NO_EXIT, "parse error in %s", errorfile);
+	}
 	return -1;
     }
     return 0;
