@@ -253,7 +253,7 @@ char *sudo_getepw	__P((const struct passwd *));
 int gettime		__P((struct timeval *));
 
 /* goodpath.c */
-char *sudo_goodpath	__P((const char *, struct stat *));
+int sudo_goodpath	__P((const char *, struct stat *));
 
 /* gram.y */
 int yyparse		__P((void));
@@ -356,6 +356,9 @@ YY_DECL;
 /* zero_bytes.c */
 void zero_bytes		__P((volatile void *, size_t));
 
+/* ttyname.c */
+char *get_process_ttyname __P((void));
+
 /* Only provide extern declarations outside of sudo.c. */
 #ifndef _SUDO_MAIN
 extern struct sudo_user sudo_user;
