@@ -136,6 +136,7 @@ struct sudo_user sudo_user;
 struct passwd *list_pw;
 static struct sudoersfile_list sudoerslist;
 static struct rbtree *alias_freelist;
+static int checkonly;
 
 int
 main(argc, argv)
@@ -144,7 +145,7 @@ main(argc, argv)
 {
     struct sudoersfile *sp;
     char *args, *editor, *sudoers_path;
-    int ch, checkonly, quiet, strict, oldperms;
+    int ch, quiet, strict, oldperms;
 #if defined(SUDO_DEVEL) && defined(__OpenBSD__)
     extern char *malloc_options;
     malloc_options = "AFGJPR";
@@ -235,12 +236,9 @@ main(argc, argv)
     /* Check edited files for a parse error and re-edit any that fail. */
     reparse_sudoers(editor, args, strict, quiet);
 
-    /* Install the sudoers temp files. */
+    /* Install the sudoers temp files as needed. */
     tq_foreach_fwd(&sudoerslist, sp) {
-	if (!sp->modified)
-	    (void) unlink(sp->tpath);
-	else
-	    (void) install_sudoers(sp, oldperms);
+	(void) install_sudoers(sp, oldperms);
     }
 
     exit(0);
@@ -533,6 +531,20 @@ install_sudoers(sp, oldperms)
 {
     struct stat sb;
 
+    if (!sp->modified) {
+	/*
+	 * No changes but fix owner/mode if needed.
+	 */
+	(void) unlink(sp->tpath);
+	if (!oldperms && fstat(sp->fd, &sb) != -1) {
+	    if (sb.st_uid != SUDOERS_UID || sb.st_gid != SUDOERS_GID)
+		(void) chown(sp->path, SUDOERS_UID, SUDOERS_GID);
+	    if ((sb.st_mode & 0777) != SUDOERS_MODE)
+		(void) chmod(sp->path, SUDOERS_MODE);
+	}
+	return TRUE;
+    }
+
     /*
      * Change mode and ownership of temp file so when
      * we move it to sp->path things are kosher.
@@ -776,7 +788,11 @@ check_syntax(sudoers_path, quiet, strict)
 	    else
 		(void) printf("parse error in %s\n", errorfile);
 	} else {
+	    struct sudoersfile *sp;
 	    (void) printf("%s: parsed OK\n", sudoers_path);
+	    tq_foreach_fwd(&sudoerslist, sp) {
+		(void) printf("%s: parsed OK\n", sp->path);
+	    }
 	}
     }
     /* Check mode and owner in strict mode. */
@@ -817,6 +833,12 @@ open_sudoers(path, doedit, keepopen)
 {
     struct sudoersfile *entry;
     FILE *fp;
+    int open_flags;
+
+    if (checkonly)
+	open_flags = O_RDONLY;
+    else
+	open_flags = O_RDWR | O_CREAT;
 
     /* Check for existing entry */
     tq_foreach_fwd(&sudoerslist, entry) {
@@ -829,7 +851,7 @@ open_sudoers(path, doedit, keepopen)
 	entry->modified = 0;
 	entry->prev = entry;
 	entry->next = NULL;
-	entry->fd = open(entry->path, O_RDWR | O_CREAT, SUDOERS_MODE);
+	entry->fd = open(entry->path, open_flags, SUDOERS_MODE);
 	entry->tpath = NULL;
 	entry->doedit = doedit;
 	if (entry->fd == -1) {
@@ -837,7 +859,7 @@ open_sudoers(path, doedit, keepopen)
 	    efree(entry);
 	    return NULL;
 	}
-	if (!lock_file(entry->fd, SUDO_TLOCK))
+	if (!checkonly && !lock_file(entry->fd, SUDO_TLOCK))
 	    errorx(1, "%s busy, try again later", entry->path);
 	if ((fp = fdopen(entry->fd, "r")) == NULL)
 	    error(1, "%s", entry->path);
@@ -1004,21 +1026,19 @@ get_hostname()
 }
 
 static int
-alias_remove_recursive(name, type, strict, quiet)
+alias_remove_recursive(name, type)
     char *name;
     int type;
-    int strict;
-    int quiet;
 {
     struct member *m;
     struct alias *a;
-    int error = 0;
+    int rval = TRUE;
 
     if ((a = alias_find(name, type)) != NULL) {
 	tq_foreach_fwd(&a->members, m) {
 	    if (m->type == ALIAS) {
-		if (!alias_remove_recursive(m->name, type, strict, quiet))
-		    error = 1;
+		if (!alias_remove_recursive(m->name, type))
+		    rval = FALSE;
 	    }
 	}
     }
@@ -1026,7 +1046,7 @@ alias_remove_recursive(name, type, strict, quiet)
     a = alias_remove(name, type);
     if (a)
 	rbinsert(alias_freelist, a);
-    return error;
+    return rval;
 }
 
 static int
@@ -1122,7 +1142,7 @@ check_aliases(strict, quiet)
 	tq_foreach_fwd(&us->users, m) {
 	    if (m->type == ALIAS) {
 		alias_seqno++;
-		if (!alias_remove_recursive(m->name, USERALIAS, strict, quiet))
+		if (!alias_remove_recursive(m->name, USERALIAS))
 		    error++;
 	    }
 	}
@@ -1130,8 +1150,7 @@ check_aliases(strict, quiet)
 	    tq_foreach_fwd(&priv->hostlist, m) {
 		if (m->type == ALIAS) {
 		    alias_seqno++;
-		    if (!alias_remove_recursive(m->name, HOSTALIAS, strict,
-			quiet))
+		    if (!alias_remove_recursive(m->name, HOSTALIAS))
 			error++;
 		}
 	    }
@@ -1139,15 +1158,13 @@ check_aliases(strict, quiet)
 		tq_foreach_fwd(&cs->runasuserlist, m) {
 		    if (m->type == ALIAS) {
 			alias_seqno++;
-			if (!alias_remove_recursive(m->name, RUNASALIAS,
-			    strict, quiet))
+			if (!alias_remove_recursive(m->name, RUNASALIAS))
 			    error++;
 		    }
 		}
 		if ((m = cs->cmnd)->type == ALIAS) {
 		    alias_seqno++;
-		    if (!alias_remove_recursive(m->name, CMNDALIAS, strict,
-			quiet))
+		    if (!alias_remove_recursive(m->name, CMNDALIAS))
 			error++;
 		}
 	    }
@@ -1174,7 +1191,7 @@ check_aliases(strict, quiet)
 	    for (m = binding; m != NULL; m = m->next) {
 		if (m->type == ALIAS) {
 		    alias_seqno++;
-		    if (!alias_remove_recursive(m->name, atype, strict, quiet))
+		    if (!alias_remove_recursive(m->name, atype))
 			error++;
 		}
 	    }
